@@ -1,6 +1,6 @@
 /*
  * DISTRHO Plugin Framework (DPF)
- * Copyright (C) 2012-2015 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2012-2016 Filipe Coelho <falktx@falktx.com>
  *
  * Permission to use, copy, modify, and/or distribute this software for any purpose with
  * or without fee is hereby granted, provided that the above copyright notice and this
@@ -17,7 +17,7 @@
 // we need this for now
 //#define PUGL_GRAB_FOCUS 1
 
-#include "../../distrho/src/DistrhoDefines.h"
+#include "../Base.hpp"
 
 #undef PUGL_HAVE_CAIRO
 #undef PUGL_HAVE_GL
