@@ -1,6 +1,6 @@
 /*
  * DISTRHO Plugin Framework (DPF)
- * Copyright (C) 2012-2015 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2012-2016 Filipe Coelho <falktx@falktx.com>
  *
  * Permission to use, copy, modify, and/or distribute this software for any purpose with
  * or without fee is hereby granted, provided that the above copyright notice and this
@@ -529,6 +529,9 @@ public:
       Write a MIDI output event.@n
       This function must only be called during run().@n
       Returns false when the host buffer is full, in which case do not call this again until the next run().
+      @note This function is not implemented yet!@n
+            It's here so that developers can prepare MIDI plugins in advance.@n
+            If you plan to use this, please report to DPF authos so it can be implemented.
     */
     bool writeMidiEvent(const MidiEvent& midiEvent) noexcept;
 #endif
@@ -549,26 +552,39 @@ protected:
     */
     virtual const char* getLabel() const = 0;
 
+   /**
+      Get an extensive comment/description about the plugin.@n
+      Optional, returns nothing by default.
+    */
+    virtual const char* getDescription() const { return ""; }
+
    /**
       Get the plugin author/maker.
     */
     virtual const char* getMaker() const = 0;
 
    /**
-      Get the plugin license name (a single line of text).@n
+      Get the plugin homepage.@n
+      Optional, returns nothing by default.
+    */
+    virtual const char* getHomePage() const { return ""; }
+
+   /**
+      Get the plugin license (a single line of text or a URL).@n
       For commercial plugins this should return some short copyright information.
     */
     virtual const char* getLicense() const = 0;
 
    /**
-      Get the plugin version, in hexadecimal.@n
-      TODO format to be defined
+      Get the plugin version, in hexadecimal.
+      @see d_version()
     */
     virtual uint32_t getVersion() const = 0;
 
    /**
       Get the plugin unique Id.@n
       This value is used by LADSPA, DSSI and VST plugin formats.
+      @see d_cconst()
     */
     virtual int64_t getUniqueId() const = 0;
 
@@ -631,6 +647,16 @@ protected:
     virtual void loadProgram(uint32_t index) = 0;
 #endif
 
+#if DISTRHO_PLUGIN_WANT_FULL_STATE
+   /**
+      Get the value of an internal state.@n
+      The host may call this function from any non-realtime context.@n
+      Must be implemented by your plugin class if DISTRHO_PLUGIN_WANT_PROGRAMS or DISTRHO_PLUGIN_WANT_FULL_STATE is enabled.
+      @note The use of this function breaks compatibility with the DSSI format.
+    */
+    virtual String getState(const char* key) const = 0;
+#endif
+
 #if DISTRHO_PLUGIN_WANT_STATE
    /**
       Change an internal state @a key to @a value.@n
